@@ -63,6 +63,14 @@ class AbstractSchema:
     def get_node_name_for_model(self, model_class):
         raise NotImplementedError()
 
+    @abstractmethod
+    def build_query_root_type(self, registry):
+        raise NotImplementedError()
+
+    @abstractmethod
+    def build_mutation_root_type(self, registry):
+        raise NotImplementedError()
+
     @abstractproperty
     def BUILDER_MAP(self):
         raise NotImplementedError()
