@@ -22,24 +22,11 @@ class QueryFieldsVisitor(Visitor):
             gql_type = gql_type.of_type
         return gql_type
 
-    # Why are we using hello/goodbye instead of enter/leave?
-    # Because we don't want GraphQL's internals to call these methods.
-    # Instead, only we should call them (see comment below in enter_field).
-    def hello_fragment_spread(self, node, *args):
-        fragment_def = self.info.fragments[node.name.value]
-        for selection in fragment_def.selection_set.selections:
-            if isinstance(selection, FragmentSpreadNode):
-                self.hello_fragment_spread(selection, *args)
-            else:
-                self.enter_field(selection, *args)
-
-    def goodbye_fragment_spread(self, node, *args):
-        fragment_def = self.info.fragments[node.name.value]
-        for selection in fragment_def.selection_set.selections:
-            if isinstance(selection, FragmentSpreadNode):
-                self.goodbye_fragment_spread(selection, *args)
-            else:
-                self.leave_field(selection, *args)
+    # By returning the referenced fragment, we instruct GraphQL to continue
+    # traversing the document contained within the fragment itself. This is
+    # necessary to ensure queries that use fragments are correctly optimized.
+    def enter_fragment_spread(self, node, *args):
+        return self.info.fragments[node.name.value]
 
     def enter_field(
         self,
@@ -100,17 +87,6 @@ class QueryFieldsVisitor(Visitor):
                     self.hello_optimizable_tree(model_class, arguments)
                 self.model_class_stack.append(model_class)
 
-            # It would be nice if we could rely on GraphQL's internals
-            # to correctly call enter/leave_fragment_spread for every fragment
-            # spread in the document. But for some reason nested fragments are
-            # skipped (see test_query_succeeds_when_using_nested_fragments).
-            # So here we manually call our own hello/goodbye_fragment_spread
-            # in order to correctly traverse the query document.
-            for selection in node.selection_set.selections:
-                if isinstance(selection, FragmentSpreadNode):
-                    self.hello_fragment_spread(selection, key, node, path, ancestors)
-                    self.goodbye_fragment_spread(selection, key, node, path, ancestors)
-
     def hello_optimizable_tree(self, model_class, arguments):
         pass
 
